( cd usr/bin ; rm -rf scons-time )
( cd usr/bin ; ln -sf scons-time-2.2.0 scons-time )
( cd usr/bin ; rm -rf scons )
( cd usr/bin ; ln -sf scons-2.2.0 scons )
( cd usr/bin ; rm -rf sconsign )
( cd usr/bin ; ln -sf sconsign-2.2.0 sconsign )
